
# Autogenerated by mlir-tblgen; don't manually edit.

from enum import IntEnum, auto, IntFlag
from ._ods_common import _cext as _ods_cext
from ..ir import register_attribute_builder
_ods_ir = _ods_cext.ir

class RcpRoundingMode(IntEnum):
    """Rounding mode of rcp"""

    APPROX = 0
    RN = 1
    RZ = 2
    RM = 3
    RP = 4

    def __str__(self):
        if self is RcpRoundingMode.APPROX:
            return "approx"
        if self is RcpRoundingMode.RN:
            return "rn"
        if self is RcpRoundingMode.RZ:
            return "rz"
        if self is RcpRoundingMode.RM:
            return "rm"
        if self is RcpRoundingMode.RP:
            return "rp"
        raise ValueError("Unknown RcpRoundingMode enum entry.")



@register_attribute_builder("RcpRoundingMode", allow_existing=True)
def _rcproundingmode(x, context):
    return _ods_ir.IntegerAttr.get(_ods_ir.IntegerType.get_signless(32, context=context), int(x))

class TensorMapInterleaveKind(IntEnum):
    """Tensor map interleave layout type"""

    INTERLEAVE_NONE = 0
    INTERLEAVE_16B = 1
    INTERLEAVE_32B = 2

    def __str__(self):
        if self is TensorMapInterleaveKind.INTERLEAVE_NONE:
            return "none"
        if self is TensorMapInterleaveKind.INTERLEAVE_16B:
            return "interleave_16b"
        if self is TensorMapInterleaveKind.INTERLEAVE_32B:
            return "interleave_32b"
        raise ValueError("Unknown TensorMapInterleaveKind enum entry.")



@register_attribute_builder("TensorMapInterleaveKind", allow_existing=True)
def _tensormapinterleavekind(x, context):
    return _ods_ir.IntegerAttr.get(_ods_ir.IntegerType.get_signless(32, context=context), int(x))

class TensorMapL2PromoKind(IntEnum):
    """Tensor map L2 promotion type"""

    L2PROMO_NONE = 0
    L2PROMO_64B = 1
    L2PROMO_128B = 2
    L2PROMO_256B = 3

    def __str__(self):
        if self is TensorMapL2PromoKind.L2PROMO_NONE:
            return "none"
        if self is TensorMapL2PromoKind.L2PROMO_64B:
            return "l2promo_64b"
        if self is TensorMapL2PromoKind.L2PROMO_128B:
            return "l2promo_128b"
        if self is TensorMapL2PromoKind.L2PROMO_256B:
            return "l2promo_256b"
        raise ValueError("Unknown TensorMapL2PromoKind enum entry.")



@register_attribute_builder("TensorMapL2PromoKind", allow_existing=True)
def _tensormapl2promokind(x, context):
    return _ods_ir.IntegerAttr.get(_ods_ir.IntegerType.get_signless(32, context=context), int(x))

class TensorMapOOBKind(IntEnum):
    """Tensor map out-of-bounds fill type"""

    OOB_ZERO = 0
    OOB_NAN = 1

    def __str__(self):
        if self is TensorMapOOBKind.OOB_ZERO:
            return "zero"
        if self is TensorMapOOBKind.OOB_NAN:
            return "nan"
        raise ValueError("Unknown TensorMapOOBKind enum entry.")



@register_attribute_builder("TensorMapOOBKind", allow_existing=True)
def _tensormapoobkind(x, context):
    return _ods_ir.IntegerAttr.get(_ods_ir.IntegerType.get_signless(32, context=context), int(x))

class TensorMapSwizzleKind(IntEnum):
    """Tensor map swizzling mode of shared memory banks"""

    SWIZZLE_NONE = 0
    SWIZZLE_32B = 1
    SWIZZLE_64B = 2
    SWIZZLE_128B = 3

    def __str__(self):
        if self is TensorMapSwizzleKind.SWIZZLE_NONE:
            return "none"
        if self is TensorMapSwizzleKind.SWIZZLE_32B:
            return "swizzle_32b"
        if self is TensorMapSwizzleKind.SWIZZLE_64B:
            return "swizzle_64b"
        if self is TensorMapSwizzleKind.SWIZZLE_128B:
            return "swizzle_128b"
        raise ValueError("Unknown TensorMapSwizzleKind enum entry.")



@register_attribute_builder("TensorMapSwizzleKind", allow_existing=True)
def _tensormapswizzlekind(x, context):
    return _ods_ir.IntegerAttr.get(_ods_ir.IntegerType.get_signless(32, context=context), int(x))

@register_attribute_builder("nvgpu.RcpRoundingModeAttr")
def _rcproundingmodeattr(x, context):
    return _ods_ir.Attribute.parse(f'#nvgpu<rcp_rounding_mode {str(x)}>', context=context)

@register_attribute_builder("nvgpu.TensorMapInterleaveAttr")
def _tensormapinterleaveattr(x, context):
    return _ods_ir.Attribute.parse(f'#nvgpu<interleave {str(x)}>', context=context)

@register_attribute_builder("nvgpu.TensorMapL2PromoAttr")
def _tensormapl2promoattr(x, context):
    return _ods_ir.Attribute.parse(f'#nvgpu<l2promo {str(x)}>', context=context)

@register_attribute_builder("nvgpu.TensorMapOOBAttr")
def _tensormapoobattr(x, context):
    return _ods_ir.Attribute.parse(f'#nvgpu<oob {str(x)}>', context=context)

@register_attribute_builder("nvgpu.TensorMapSwizzleAttr")
def _tensormapswizzleattr(x, context):
    return _ods_ir.Attribute.parse(f'#nvgpu<swizzle {str(x)}>', context=context)




#  Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
#  See https://llvm.org/LICENSE.txt for license information.
#  SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

from .._structured_transform_ops_gen import *
from .._structured_transform_ops_gen import _Dialect
from .._structured_transform_enum_gen import *

try:
    from ...ir import *
    from ...dialects import transform
    from .._ods_common import (
        DynamicIndexList,
        IntOrAttrList,
        MixedValues,
        OptionalBoolList,
        OptionalIntList,
        _cext as _ods_cext,
        _dispatch_dynamic_index_list,
        _dispatch_mixed_values,
        _get_int_array_array_attr,
        _get_int_array_attr,
        _get_value_list,
        _get_value_or_attribute_value,
    )
except ImportError as e:
    raise RuntimeError("Error loading imports from extension module") from e

from typing import List, Optional, Sequence, Union, overload


@_ods_cext.register_operation(_Dialect, replace=True)
class BufferizeToAllocationOp(BufferizeToAllocationOp):
    """Specialization for BufferizeToAllocationOp class."""

    def __init__(
        self,
        target: Union[Operation, OpView, Value],
        *,
        memory_space: Optional[Union[int, str, Attribute]] = None,
        memcpy_op: Optional[str] = None,
        alloc_op: Optional[str] = None,
        bufferize_destination_only: Optional[bool] = None,
        loc=None,
        ip=None,
    ):
        if isinstance(memory_space, int):
            memory_space = str(memory_space)
        if isinstance(memory_space, str):
            memory_space = Attribute.parse(memory_space)

        super().__init__(
            target,
            memory_space=memory_space,
            memcpy_op=memcpy_op,
            alloc_op=alloc_op,
            bufferize_destination_only=bufferize_destination_only,
            loc=loc,
            ip=ip,
        )


@_ods_cext.register_operation(_Dialect, replace=True)
class DecomposeOp(DecomposeOp):
    """Specialization for DecomposeOp class."""

    def __init__(self, target: Union[Operation, Value], *, loc=None, ip=None):
        transformed_type = transform.AnyOpType.get()
        super().__init__(transformed_type, target, loc=loc, ip=ip)


@_ods_cext.register_operation(_Dialect, replace=True)
class FuseIntoContainingOp(FuseIntoContainingOp):
    """Specialization for FuseIntoContainingOp class."""

    @overload
    def __init__(
        self,
        fused_op_type: Type,
        new_containing_op_type: Type,
        producer_op: Union[Operation, OpView, Value],
        containing_op: Union[Operation, OpView, Value],
        *,
        loc=None,
        ip=None,
    ):
        ...

    @overload
    def __init__(
        self,
        producer_op: Union[Operation, OpView, Value],
        containing_op: Union[Operation, OpView, Value],
        *,
        loc=None,
        ip=None,
    ):
        ...

    def __init__(
        self,
        fused_op_type_or_producer_op: Union[Operation, OpView, Type, Value],
        new_containing_op_type_or_containing_op: Union[Operation, OpView, Type, Value],
        producer_op_or_none: Optional[Union[Operation, OpView, Value]] = None,
        containing_op_or_none: Optional[Union[Operation, OpView, Value]] = None,
        *,
        loc=None,
        ip=None,
    ):
        if isinstance(fused_op_type_or_producer_op, Type):
            if not isinstance(new_containing_op_type_or_containing_op, Type):
                raise TypeError(
                    "If 'fused_op_type_or_producer_op' is a type, then "
                    "'new_containing_op_type_or_containing_op' is expected "
                    "to be one as well."
                )
            fused_op_type = fused_op_type_or_producer_op
            new_containing_op_type = new_containing_op_type_or_containing_op
            producer_op = producer_op_or_none
            containing_op = containing_op_or_none
        else:
            fused_op_type = transform.AnyOpType.get()
            new_containing_op_type = transform.AnyOpType.get()
            producer_op = fused_op_type_or_producer_op
            containing_op = new_containing_op_type_or_containing_op

        super().__init__(
            fused_op_type,
            new_containing_op_type,
            producer_op,
            containing_op,
            loc=loc,
            ip=ip,
        )


@_ods_cext.register_operation(_Dialect, replace=True)
class FuseOp(FuseOp):
    """Specialization for FuseOp class."""

    @overload
    def __init__(
        self,
        loop_types: Union[Type, Sequence[Type]],
        target: Union[Operation, Value, OpView],
        *,
        tile_sizes: Optional[MixedValues] = None,
        tile_interchange: Optional[MixedValues] = None,
        apply_cleanup: bool = False,
        use_forall: bool = False,
        loc=None,
        ip=None,
    ):
        ...

    @overload
    def __init__(
        self,
        target: Union[Operation, Value, OpView],
        *,
        tile_sizes: Optional[MixedValues] = None,
        tile_interchange: Optional[MixedValues] = None,
        apply_cleanup: bool = False,
        use_forall: bool = False,
        loc=None,
        ip=None,
    ):
        ...

    def __init__(
        self,
        loop_types_or_target: Union[Type, Sequence[Type], Operation, OpView, Value],
        target_or_none: Optional[Union[Operation, Value, OpView]] = None,
        *,
        tile_sizes: Optional[MixedValues] = None,
        tile_interchange: Optional[MixedValues] = None,
        apply_cleanup: bool = False,
        use_forall: bool = False,
        loc=None,
        ip=None,
    ):
        tile_sizes = tile_sizes if tile_sizes else []
        tile_interchange = tile_interchange if tile_interchange else []
        (
            dynamic_tile_sizes,
            packed_tile_sizes,
            static_tile_sizes,
        ) = _dispatch_mixed_values(tile_sizes)
        (
            dynamic_tile_interchange,
            static_tile_interchange,
            _,
        ) = _dispatch_dynamic_index_list(tile_interchange)
        num_loops = (
            1
            if use_forall or packed_tile_sizes is not None
            else sum(1 for v in static_tile_sizes if v != 0)
        )

        if isinstance(loop_types_or_target, (Operation, Value, OpView)):
            loop_types = [transform.AnyOpType.get()] * num_loops
            target = loop_types_or_target
            assert target_or_none is None, "Cannot construct FuseOp with two targets."
        else:
            loop_types = (
                ([loop_types_or_target] * num_loops)
                if isinstance(loop_types_or_target, Type)
                else loop_types_or_target
            )
            target = target_or_none
        super().__init__(
            target.type,
            loop_types,
            target,
            tile_sizes=dynamic_tile_sizes,
            tile_interchange=dynamic_tile_interchange,
            packed_tile_sizes=packed_tile_sizes,
            static_tile_sizes=static_tile_sizes,
            static_tile_interchange=static_tile_interchange,
            apply_cleanup=apply_cleanup,
            use_forall=use_forall,
            loc=loc,
            ip=ip,
        )


@_ods_cext.register_operation(_Dialect, replace=True)
class GeneralizeOp(GeneralizeOp):
    """Specialization for GeneralizeOp class."""

    def __init__(self, target: Union[Operation, Value], *, loc=None, ip=None):
        transformed_type = transform.AnyOpType.get()
        super().__init__(transformed_type, target, loc=loc, ip=ip)


@_ods_cext.register_operation(_Dialect, replace=True)
class InterchangeOp(InterchangeOp):
    """Specialization for InterchangeOp class."""

    def __init__(
        self,
        target: Union[Operation, Value],
        *,
        iterator_interchange: OptionalIntList = None,
        loc=None,
        ip=None,
    ):
        transformed_type = transform.AnyOpType.get()
        super().__init__(
            transformed_type,
            target,
            iterator_interchange=iterator_interchange,
            loc=loc,
            ip=ip,
        )


@_ods_cext.register_operation(_Dialect, replace=True)
class MapCopyToThreadsOp(MapCopyToThreadsOp):
    """Specialization for MapCopyToThreadsOp class."""

    @overload
    def __init__(
        self,
        forall_op_type: Type,
        tiled_op_type: Type,
        target: Union[Operation, OpView, Value],
        *,
        total_num_threads: Union[int, IntegerAttr],
        desired_bit_alignment: Union[int, IntegerAttr],
        loc=None,
        ip=None,
    ):
        ...

    @overload
    def __init__(
        self,
        target: Union[Operation, OpView, Value],
        *,
        total_num_threads: Union[int, IntegerAttr],
        desired_bit_alignment: Union[int, IntegerAttr],
        loc=None,
        ip=None,
    ):
        ...

    def __init__(
        self,
        forall_op_type_or_target: Union[Operation, OpView, Type, Value],
        tiled_op_type_or_none: Optional[Type] = None,
        target_or_none: Optional[Union[Operation, OpView, Value]] = None,
        *,
        total_num_threads: Union[int, IntegerAttr],
        desired_bit_alignment: Union[int, IntegerAttr],
        loc=None,
        ip=None,
    ):
        if isinstance(forall_op_type_or_target, Type):
            forall_op_type = forall_op_type_or_target
            tiled_op_type = tiled_op_type_or_none
            target = target_or_none
        else:
            forall_op_type = transform.AnyOpType.get()
            tiled_op_type = transform.AnyOpType.get()
            target = forall_op_type_or_target

        super().__init__(
            forall_op_type,
            tiled_op_type,
            target,
            total_num_threads=total_num_threads,
            desired_bit_alignment=desired_bit_alignment,
            loc=loc,
            ip=ip,
        )


@_ods_cext.register_operation(_Dialect, replace=True)
class VectorizeOp(VectorizeOp):
    """Specialization for VectorizeOp class."""

    def __init__(
        self,
        target: Union[Operation, OpView, Value],
        vector_sizes: Optional[Union[DynamicIndexList, ArrayAttr]] = None,
        *,
        vectorize_nd_extract: Optional[bool] = None,
        scalable_sizes: OptionalBoolList = None,
        static_vector_sizes: OptionalIntList = None,
        loc=None,
        ip=None,
    ):
        if (
            scalable_sizes is None
            and static_vector_sizes is None
            and vector_sizes is None
        ):
            dynamic_vector_sizes = []
        elif scalable_sizes is None and static_vector_sizes is None:
            (
                dynamic_vector_sizes,
                static_vector_sizes,
                scalable_sizes,
            ) = _dispatch_dynamic_index_list(vector_sizes)
        elif scalable_sizes is None or static_vector_sizes is None:
            raise TypeError(
                "'scalable_sizes' and 'static_vector_sizes' must either both "
                "be given explicitly or both be given as part of 'vector_sizes'."
            )
        else:
            dynamic_vector_sizes = vector_sizes

        super().__init__(
            target,
            vector_sizes=dynamic_vector_sizes,
            static_vector_sizes=static_vector_sizes,
            scalable_sizes=scalable_sizes,
            vectorize_nd_extract=vectorize_nd_extract,
            loc=loc,
            ip=ip,
        )


@_ods_cext.register_operation(_Dialect, replace=True)
class MatchOp(MatchOp):
    """Specialization for MatchOp class."""

    @overload
    @classmethod
    def match_op_names(
        cls,
        target: Union[Operation, Value],
        names: Union[str, Sequence[str]],
        *,
        loc=None,
        ip=None,
    ):
        ...

    @overload
    @classmethod
    def match_op_names(
        cls,
        result_type: Type,
        target: Union[Operation, Value],
        names: Union[str, Sequence[str]],
        *,
        loc=None,
        ip=None,
    ):
        ...

    @classmethod
    def match_op_names(
        cls,
        result_type_or_target: Union[Type, Operation, Value],
        target_or_names: Union[Operation, Value, Sequence[str], str],
        names_or_none: Optional[Union[Sequence[str], str]] = None,
        *,
        loc=None,
        ip=None,
    ):
        if isinstance(result_type_or_target, Type):
            result_type = result_type_or_target
            target = target_or_names
            names = names_or_none
        else:
            result_type = transform.AnyOpType.get()
            target = result_type_or_target
            names = target_or_names

        if isinstance(names, str):
            names = [names]

        return cls(
            result_type,
            target,
            ops=ArrayAttr.get(list(map(lambda s: StringAttr.get(s), names))),
            loc=loc,
            ip=ip,
        )


@_ods_cext.register_operation(_Dialect, replace=True)
class MultiTileSizesOp(MultiTileSizesOp):
    """Specialization for MultiTileSizesOp class."""

    def __init__(
        self,
        result_type: Type,
        target: Union[Operation, Value],
        *,
        dimension: Union[int, IntegerAttr],
        target_size: Union[int, IntegerAttr],
        divisor: Optional[Optional[Union[int, IntegerAttr]]] = None,
        loc=None,
        ip=None,
    ):
        super().__init__(
            result_type,
            result_type,
            result_type,
            target,
            dimension=dimension,
            target_size=target_size,
            divisor=divisor,
            loc=loc,
            ip=ip,
        )


@_ods_cext.register_operation(_Dialect, replace=True)
class PadOp(PadOp):
    """Specialization for PadOp class."""

    def __init__(
        self,
        target: Union[Operation, OpView, Value],
        *,
        pad_to_multiple_of: Optional[Union[DynamicIndexList, ArrayAttr]] = None,
        padding_values: Optional[Union[ArrayAttr, Sequence[Attribute]]] = None,
        padding_dimensions: OptionalIntList = None,
        nofold_flags: OptionalIntList = None,
        transpose_paddings: Optional[
            Union[ArrayAttr, Sequence[Union[ArrayAttr, IntOrAttrList]]]
        ] = None,
        copy_back_op: Optional[Union[str, StringAttr]] = None,
        loc=None,
        ip=None,
    ):
        if pad_to_multiple_of is None:
            dynamic_pad_to_multiple_of = []
            static_pad_to_multiple_of = None
        else:
            (
                dynamic_pad_to_multiple_of,
                static_pad_to_multiple_of,
                _,
            ) = _dispatch_dynamic_index_list(pad_to_multiple_of)

        transpose_paddings = _get_int_array_array_attr(transpose_paddings)

        any_op_type = transform.AnyOpType.get()
        super().__init__(
            any_op_type,
            any_op_type,
            any_op_type,
            target,
            pad_to_multiple_of=dynamic_pad_to_multiple_of,
            padding_values=padding_values,
            padding_dimensions=padding_dimensions,
            static_pad_to_multiple_of=static_pad_to_multiple_of,
            nofold_flags=nofold_flags,
            transpose_paddings=transpose_paddings,
            copy_back_op=copy_back_op,
            loc=loc,
            ip=ip,
        )


@_ods_cext.register_operation(_Dialect, replace=True)
class ScalarizeOp(ScalarizeOp):
    """Specialization for ScalarizeOp class."""

    def __init__(self, target: Union[Operation, Value], *, loc=None, ip=None):
        result_type = transform.AnyOpType.get()
        super().__init__(result_type, target, loc=loc, ip=ip)


@_ods_cext.register_operation(_Dialect, replace=True)
class SplitOp(SplitOp):
    """Specialization for SplitOp class."""

    def __init__(
        self,
        target: Union[Operation, Value],
        dimension: Union[int, Attribute],
        chunk_sizes: Union[int, Operation, Value, Attribute],
        *,
        loc=None,
        ip=None,
    ):
        if isinstance(chunk_sizes, int):
            static_chunk_sizes = chunk_sizes
            dynamic_chunk_sizes = None
        else:
            static_chunk_sizes = ShapedType.get_dynamic_size()
            dynamic_chunk_sizes = chunk_sizes

        super().__init__(
            target.type,
            target,
            dimension=dimension,
            static_chunk_sizes=static_chunk_sizes,
            dynamic_chunk_sizes=dynamic_chunk_sizes,
            loc=loc,
            ip=ip,
        )


@_ods_cext.register_operation(_Dialect, replace=True)
class TileUsingForOp(TileUsingForOp):
    """Specialization for TileUsingForOp class."""

    @overload
    def __init__(
        self,
        loop_types: Union[Type, List[Type]],
        target: Union[Operation, Value],
        *,
        sizes: Optional[Union[DynamicIndexList, ArrayAttr]] = None,
        interchange: OptionalIntList = None,
        loc=None,
        ip=None,
    ):
        ...

    @overload
    def __init__(
        self,
        target: Union[Operation, Value, OpView],
        *,
        sizes: Optional[Union[DynamicIndexList, ArrayAttr]] = None,
        interchange: OptionalIntList = None,
        loc=None,
        ip=None,
    ):
        ...

    def __init__(
        self,
        loop_types_or_target: Union[Type, List[Type], Operation, Value],
        target_or_none: Optional[Union[Operation, Value, OpView]] = None,
        *,
        sizes: Optional[Union[DynamicIndexList, ArrayAttr]] = None,
        interchange: OptionalIntList = None,
        loc=None,
        ip=None,
    ):
        (
            dynamic_sizes,
            static_sizes,
            scalable_sizes,
        ) = _dispatch_dynamic_index_list(sizes)

        num_loops = sum(v if v == 0 else 1 for v in static_sizes)

        if isinstance(loop_types_or_target, (Operation, Value, OpView)):
            loop_types = [transform.AnyOpType.get()] * num_loops
            target = loop_types_or_target
            assert (
                target_or_none is None
            ), "Cannot construct TileUsingForOp with two targets."
        else:
            loop_types = (
                ([loop_types_or_target] * num_loops)
                if isinstance(loop_types_or_target, Type)
                else loop_types_or_target
            )
            target = target_or_none

        super().__init__(
            target.type,
            loop_types,
            target,
            dynamic_sizes=dynamic_sizes,
            static_sizes=static_sizes,
            interchange=interchange,
            scalable_sizes=scalable_sizes,
            loc=loc,
            ip=ip,
        )


@_ods_cext.register_operation(_Dialect, replace=True)
class TileUsingForallOp(TileUsingForallOp):
    """Specialization for TileUsingForallOp class."""

    @overload
    def __init__(
        self,
        loops_type: Type,
        tiled_op_type: Type,
        target: Union[Operation, Value, OpView],
        *,
        num_threads: Optional[MixedValues] = None,
        tile_sizes: MixedValues = None,
        mapping=None,
        loc=None,
        ip=None,
    ):
        ...

    @overload
    def __init__(
        self,
        target: Union[Operation, Value, OpView],
        *,
        num_threads: Optional[MixedValues] = None,
        tile_sizes: MixedValues = None,
        mapping=None,
        loc=None,
        ip=None,
    ):
        ...

    def __init__(
        self,
        loops_type_or_target: Union[
            Type, Union[Operation, Value, OpView]  # loops_type
        ],  # target
        tiled_op_type_or_none: Optional[Type] = None,
        target_or_none: Optional[Union[Operation, Value, OpView]] = None,
        *,
        num_threads: MixedValues = None,
        tile_sizes: MixedValues = None,
        mapping=None,
        loc=None,
        ip=None,
    ):
        # `Type` arguments in the front are optional: add default values to front.
        if isinstance(loops_type_or_target, Type):
            # First overload: type arguments provided.
            if not isinstance(tiled_op_type_or_none, Type):
                raise TypeError(
                    "If 'loops_type_or_target' is a type, then "
                    "'tiled_op_type_or_none' is expected to be one as well."
                )
            loops_type = loops_type_or_target
            tiled_op_type = tiled_op_type_or_none
            target = target_or_none
        else:
            # Last overload: type arguments missing.
            loops_type = transform.AnyOpType.get()
            tiled_op_type = transform.AnyOpType.get()
            target = loops_type_or_target

        # Unpack mixed num_threads.
        (
            dynamic_num_threads,
            packed_num_threads,
            num_threads_attr,
        ) = _dispatch_mixed_values(num_threads)

        # Unpack mixed tile_sizes.
        (
            dynamic_tile_sizes,
            packed_tile_sizes,
            tile_sizes_attr,
        ) = _dispatch_mixed_values(tile_sizes)

        super().__init__(
            loops_type,
            tiled_op_type,
            target=target,
            tile_sizes=dynamic_tile_sizes,
            packed_tile_sizes=packed_tile_sizes,
            static_tile_sizes=tile_sizes_attr,
            num_threads=dynamic_num_threads,
            packed_num_threads=packed_num_threads,
            static_num_threads=num_threads_attr,
            mapping=mapping,
            loc=loc,
            ip=ip,
        )


@_ods_cext.register_operation(_Dialect, replace=True)
class VectorizeChildrenAndApplyPatternsOp(VectorizeChildrenAndApplyPatternsOp):
    """Specialization for VectorizeChildrenAndApplyPatternsOp class."""

    def __init__(
        self,
        target: Union[Operation, Value],
        *,
        disable_multi_reduction_to_contract_patterns: bool = False,
        disable_transfer_permutation_map_lowering_patterns: bool = False,
        vectorize_nd_extract: bool = False,
        vectorize_padding: bool = False,
        flatten_1d_depthwise_conv: bool = False,
        fold_type_extensions_into_contract: bool = False,
        loc=None,
        ip=None,
    ):
        transformed_type = transform.AnyOpType.get()
        super().__init__(
            transformed_type,
            target,
            disable_multi_reduction_to_contract_patterns=disable_multi_reduction_to_contract_patterns,
            disable_transfer_permutation_map_lowering_patterns=disable_transfer_permutation_map_lowering_patterns,
            flatten_1d_depthwise_conv=flatten_1d_depthwise_conv,
            vectorize_nd_extract=vectorize_nd_extract,
            vectorize_padding=vectorize_padding,
            fold_type_extensions_into_contract=fold_type_extensions_into_contract,
            loc=loc,
            ip=ip,
        )




# Autogenerated by mlir-tblgen; don't manually edit.

from ._ods_common import _cext as _ods_cext
from ._ods_common import (
    equally_sized_accessor as _ods_equally_sized_accessor,
    get_default_loc_context as _ods_get_default_loc_context,
    get_op_results_or_values as _get_op_results_or_values,
    segmented_accessor as _ods_segmented_accessor,
)
_ods_ir = _ods_cext.ir
_ods_cext.globals.register_traceback_file_exclusion(__file__)

import builtins
from typing import Any as _Any, Sequence as _Sequence, Union as _Union, Optional as _Optional
import sys as _sys
if _sys.version_info >= (3, 12):
  from collections.abc import Buffer as _Buffer  # pytype: disable=not-supported-yet
else:
  try:
    from typing_extensions import Buffer as _Buffer
  except ImportError:
    _Buffer = _Any


from ._transform_ops_gen import _Dialect

@_ods_cext.register_operation(_Dialect)
class ApplyShuffleVectorFMAOpsPatternsOp(_ods_ir.OpView):
  r"""
  Collect patterns to shuffle FMAs with x86 operations as operands 
  such that FMAs are grouped with respect to odd/even packed index.
  """

  OPERATION_NAME = "transform.apply_patterns.x86.shuffle_vector_fma_ops"

  _ODS_REGIONS = (0, True)

  def __init__(self, *, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None):
    operands = []
    attributes = {}
    regions = None
    _ods_context = _ods_get_default_loc_context(loc)
    results = []
    _ods_successors = None
    super().__init__(self.OPERATION_NAME, self._ODS_REGIONS, self._ODS_OPERAND_SEGMENTS, self._ODS_RESULT_SEGMENTS, attributes=attributes, results=results, operands=operands, successors=_ods_successors, regions=regions, loc=loc, ip=ip)

@_ods_cext.register_op_adaptor(ApplyShuffleVectorFMAOpsPatternsOp)
class ApplyShuffleVectorFMAOpsPatternsOpAdaptor(_ods_ir.OpAdaptor):
  OPERATION_NAME = "transform.apply_patterns.x86.shuffle_vector_fma_ops"

def apply_patterns_x86_shuffle_vector_fma_ops(*, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None) -> ApplyShuffleVectorFMAOpsPatternsOp:
  return ApplyShuffleVectorFMAOpsPatternsOp(loc=loc, ip=ip)

@_ods_cext.register_operation(_Dialect)
class ApplySinkVectorProducerOpsPatternsOp(_ods_ir.OpView):
  r"""
  Collect patterns to sink vector producer operations forward in a block to 
       place them immediately before their first use.
  """

  OPERATION_NAME = "transform.apply_patterns.x86.sink_vector_producer_ops"

  _ODS_REGIONS = (0, True)

  def __init__(self, *, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None):
    operands = []
    attributes = {}
    regions = None
    _ods_context = _ods_get_default_loc_context(loc)
    results = []
    _ods_successors = None
    super().__init__(self.OPERATION_NAME, self._ODS_REGIONS, self._ODS_OPERAND_SEGMENTS, self._ODS_RESULT_SEGMENTS, attributes=attributes, results=results, operands=operands, successors=_ods_successors, regions=regions, loc=loc, ip=ip)

@_ods_cext.register_op_adaptor(ApplySinkVectorProducerOpsPatternsOp)
class ApplySinkVectorProducerOpsPatternsOpAdaptor(_ods_ir.OpAdaptor):
  OPERATION_NAME = "transform.apply_patterns.x86.sink_vector_producer_ops"

def apply_patterns_x86_sink_vector_producer_ops(*, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None) -> ApplySinkVectorProducerOpsPatternsOp:
  return ApplySinkVectorProducerOpsPatternsOp(loc=loc, ip=ip)

@_ods_cext.register_operation(_Dialect)
class ApplyVectorContractBF16ToFMAPatternsOp(_ods_ir.OpView):
  r"""
  Collect patterns to lower a BF16 type vector.contract operation
      to a FMA via emulation lowering using BF16 packed operations.
  """

  OPERATION_NAME = "transform.apply_patterns.x86.vector_contract_bf16_to_fma"

  _ODS_REGIONS = (0, True)

  def __init__(self, *, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None):
    operands = []
    attributes = {}
    regions = None
    _ods_context = _ods_get_default_loc_context(loc)
    results = []
    _ods_successors = None
    super().__init__(self.OPERATION_NAME, self._ODS_REGIONS, self._ODS_OPERAND_SEGMENTS, self._ODS_RESULT_SEGMENTS, attributes=attributes, results=results, operands=operands, successors=_ods_successors, regions=regions, loc=loc, ip=ip)

@_ods_cext.register_op_adaptor(ApplyVectorContractBF16ToFMAPatternsOp)
class ApplyVectorContractBF16ToFMAPatternsOpAdaptor(_ods_ir.OpAdaptor):
  OPERATION_NAME = "transform.apply_patterns.x86.vector_contract_bf16_to_fma"

def apply_patterns_x86_vector_contract_bf16_to_fma(*, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None) -> ApplyVectorContractBF16ToFMAPatternsOp:
  return ApplyVectorContractBF16ToFMAPatternsOp(loc=loc, ip=ip)

@_ods_cext.register_operation(_Dialect)
class ApplyVectorContractToAMXDotProductPatternsOp(_ods_ir.OpView):
  r"""
     Collect patterns to lower a BF16/Int8 type vector.contract operation 
  to a BF16/Int8 AMX tiled dot-product.
  """

  OPERATION_NAME = "transform.apply_patterns.x86.vector_contract_to_amx_dot_product"

  _ODS_REGIONS = (0, True)

  def __init__(self, *, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None):
    operands = []
    attributes = {}
    regions = None
    _ods_context = _ods_get_default_loc_context(loc)
    results = []
    _ods_successors = None
    super().__init__(self.OPERATION_NAME, self._ODS_REGIONS, self._ODS_OPERAND_SEGMENTS, self._ODS_RESULT_SEGMENTS, attributes=attributes, results=results, operands=operands, successors=_ods_successors, regions=regions, loc=loc, ip=ip)

@_ods_cext.register_op_adaptor(ApplyVectorContractToAMXDotProductPatternsOp)
class ApplyVectorContractToAMXDotProductPatternsOpAdaptor(_ods_ir.OpAdaptor):
  OPERATION_NAME = "transform.apply_patterns.x86.vector_contract_to_amx_dot_product"

def apply_patterns_x86_vector_contract_to_amx_dot_product(*, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None) -> ApplyVectorContractToAMXDotProductPatternsOp:
  return ApplyVectorContractToAMXDotProductPatternsOp(loc=loc, ip=ip)

@_ods_cext.register_operation(_Dialect)
class ApplyVectorContractToFMAPatternsOp(_ods_ir.OpView):
  r"""
  Collect patterns to lower a F32 type vector.contract operation to a FMA.
  """

  OPERATION_NAME = "transform.apply_patterns.x86.vector_contract_to_fma"

  _ODS_REGIONS = (0, True)

  def __init__(self, *, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None):
    operands = []
    attributes = {}
    regions = None
    _ods_context = _ods_get_default_loc_context(loc)
    results = []
    _ods_successors = None
    super().__init__(self.OPERATION_NAME, self._ODS_REGIONS, self._ODS_OPERAND_SEGMENTS, self._ODS_RESULT_SEGMENTS, attributes=attributes, results=results, operands=operands, successors=_ods_successors, regions=regions, loc=loc, ip=ip)

@_ods_cext.register_op_adaptor(ApplyVectorContractToFMAPatternsOp)
class ApplyVectorContractToFMAPatternsOpAdaptor(_ods_ir.OpAdaptor):
  OPERATION_NAME = "transform.apply_patterns.x86.vector_contract_to_fma"

def apply_patterns_x86_vector_contract_to_fma(*, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None) -> ApplyVectorContractToFMAPatternsOp:
  return ApplyVectorContractToFMAPatternsOp(loc=loc, ip=ip)

@_ods_cext.register_operation(_Dialect)
class ApplyVectorContractToPackedTypeDotProductPatternsOp(_ods_ir.OpView):
  r"""
     Collect patterns to lower a BF16/Int8 type vector.contract operation 
  to a BF16/Int8 dot-product.
  """

  OPERATION_NAME = "transform.apply_patterns.x86.vector_contract_to_packed_type_dot_product"

  _ODS_REGIONS = (0, True)

  def __init__(self, *, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None):
    operands = []
    attributes = {}
    regions = None
    _ods_context = _ods_get_default_loc_context(loc)
    results = []
    _ods_successors = None
    super().__init__(self.OPERATION_NAME, self._ODS_REGIONS, self._ODS_OPERAND_SEGMENTS, self._ODS_RESULT_SEGMENTS, attributes=attributes, results=results, operands=operands, successors=_ods_successors, regions=regions, loc=loc, ip=ip)

@_ods_cext.register_op_adaptor(ApplyVectorContractToPackedTypeDotProductPatternsOp)
class ApplyVectorContractToPackedTypeDotProductPatternsOpAdaptor(_ods_ir.OpAdaptor):
  OPERATION_NAME = "transform.apply_patterns.x86.vector_contract_to_packed_type_dot_product"

def apply_patterns_x86_vector_contract_to_packed_type_dot_product(*, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None) -> ApplyVectorContractToPackedTypeDotProductPatternsOp:
  return ApplyVectorContractToPackedTypeDotProductPatternsOp(loc=loc, ip=ip)



#  Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
#  See https://llvm.org/LICENSE.txt for license information.
#  SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception

from ._llvm_ops_gen import *
from ._llvm_ops_gen import _Dialect
from ._llvm_enum_gen import *
from .._mlir_libs._mlirDialectsLLVM import *
from ..ir import Value, IntegerType, IntegerAttr
from ._ods_common import get_op_result_or_op_results as _get_op_result_or_op_results


def mlir_constant(value, *, loc=None, ip=None) -> Value:
    return _get_op_result_or_op_results(
        ConstantOp(res=value.type, value=value, loc=loc, ip=ip)
    )


def md_const(val, *, width=32, context=None):
    if not isinstance(val, int):
        raise NotImplementedError(
            f"{val=} not supported; only integers currently supported."
        )
    i_type = IntegerType.get_signless(width, context=context)
    return MDConstantAttr.get(IntegerAttr.get(i_type, val), context=context)


def md_str(s, *, context=None):
    return MDStringAttr.get(s, context=context)




# Autogenerated by mlir-tblgen; don't manually edit.

from ._ods_common import _cext as _ods_cext
from ._ods_common import (
    equally_sized_accessor as _ods_equally_sized_accessor,
    get_default_loc_context as _ods_get_default_loc_context,
    get_op_results_or_values as _get_op_results_or_values,
    segmented_accessor as _ods_segmented_accessor,
)
_ods_ir = _ods_cext.ir
_ods_cext.globals.register_traceback_file_exclusion(__file__)

import builtins
from typing import Any as _Any, Sequence as _Sequence, Union as _Union, Optional as _Optional
import sys as _sys
if _sys.version_info >= (3, 12):
  from collections.abc import Buffer as _Buffer  # pytype: disable=not-supported-yet
else:
  try:
    from typing_extensions import Buffer as _Buffer
  except ImportError:
    _Buffer = _Any


from ._transform_ops_gen import _Dialect

@_ods_cext.register_operation(_Dialect)
class ApplyAllocToAllocaOp(_ods_ir.OpView):
  r"""
  Collects patterns to rewrite scoped dynamic allocation (`alloc`/`dealloc`
  pairs) into automatic allocation (`alloca`) in the same scope, for memrefs
  of static shape.
  
  The `size_limit` attribute controls the maximum allocated memory (in bytes,
  subject to data layout) for which the pattern applies.
  """

  OPERATION_NAME = "transform.apply_patterns.memref.alloc_to_alloca"

  _ODS_REGIONS = (0, True)

  def __init__(self, *, size_limit: _Optional[_Union[int, _ods_ir.IntegerAttr]] = None, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None):
    operands = []
    attributes = {}
    regions = None
    _ods_context = _ods_get_default_loc_context(loc)
    if size_limit is not None: attributes["size_limit"] = (size_limit if (
        isinstance(size_limit, _ods_ir.Attribute) or
        not _ods_ir.AttrBuilder.contains('I64Attr')) else
          _ods_ir.AttrBuilder.get('I64Attr')(size_limit, context=_ods_context))
    results = []
    _ods_successors = None
    super().__init__(self.OPERATION_NAME, self._ODS_REGIONS, self._ODS_OPERAND_SEGMENTS, self._ODS_RESULT_SEGMENTS, attributes=attributes, results=results, operands=operands, successors=_ods_successors, regions=regions, loc=loc, ip=ip)

  @builtins.property
  def size_limit(self) -> _Optional[_ods_ir.IntegerAttr]:
    if "size_limit" not in self.operation.attributes:
      return None
    return self.operation.attributes["size_limit"]

  @size_limit.setter
  def size_limit(self, value: _Optional[_ods_ir.IntegerAttr]):
    if value is not None:
      self.operation.attributes["size_limit"] = value
    elif "size_limit" in self.operation.attributes:
      del self.operation.attributes["size_limit"]

  @size_limit.deleter
  def size_limit(self):
    del self.operation.attributes["size_limit"]

@_ods_cext.register_op_adaptor(ApplyAllocToAllocaOp)
class ApplyAllocToAllocaOpAdaptor(_ods_ir.OpAdaptor):
  OPERATION_NAME = "transform.apply_patterns.memref.alloc_to_alloca"

  @builtins.property
  def size_limit(self) -> _Optional[_ods_ir.IntegerAttr]:
    if "size_limit" not in self.attributes:
      return None
    return self.attributes["size_limit"]

def apply_patterns_memref_alloc_to_alloca(*, size_limit: _Optional[_Union[int, _ods_ir.IntegerAttr]] = None, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None) -> ApplyAllocToAllocaOp:
  return ApplyAllocToAllocaOp(size_limit=size_limit, loc=loc, ip=ip)

@_ods_cext.register_operation(_Dialect)
class ApplyExpandOpsPatternsOp(_ods_ir.OpView):
  r"""
  Collects patterns to rewrite ops within the memref dialect.
  
  - Converts `atomic_rmw` that cannot be lowered to a simple atomic op with
    AtomicRMWOpLowering pattern, e.g. with "minf" or "maxf" attributes, to
    `memref.generic_atomic_rmw` with the expanded code.
  - Converts `memref.reshape` that has a target shape of a statically-known
    size to `memref.reinterpret_cast`.
  """

  OPERATION_NAME = "transform.apply_patterns.memref.expand_ops"

  _ODS_REGIONS = (0, True)

  def __init__(self, *, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None):
    operands = []
    attributes = {}
    regions = None
    _ods_context = _ods_get_default_loc_context(loc)
    results = []
    _ods_successors = None
    super().__init__(self.OPERATION_NAME, self._ODS_REGIONS, self._ODS_OPERAND_SEGMENTS, self._ODS_RESULT_SEGMENTS, attributes=attributes, results=results, operands=operands, successors=_ods_successors, regions=regions, loc=loc, ip=ip)

@_ods_cext.register_op_adaptor(ApplyExpandOpsPatternsOp)
class ApplyExpandOpsPatternsOpAdaptor(_ods_ir.OpAdaptor):
  OPERATION_NAME = "transform.apply_patterns.memref.expand_ops"

def apply_patterns_memref_expand_ops(*, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None) -> ApplyExpandOpsPatternsOp:
  return ApplyExpandOpsPatternsOp(loc=loc, ip=ip)

@_ods_cext.register_operation(_Dialect)
class ApplyExpandStridedMetadataPatternsOp(_ods_ir.OpView):
  r"""
  Collects patterns for expanding memref operations that modify the metadata
  (sizes, offset, strides) of a memref into easier to analyze constructs.
  """

  OPERATION_NAME = "transform.apply_patterns.memref.expand_strided_metadata"

  _ODS_REGIONS = (0, True)

  def __init__(self, *, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None):
    operands = []
    attributes = {}
    regions = None
    _ods_context = _ods_get_default_loc_context(loc)
    results = []
    _ods_successors = None
    super().__init__(self.OPERATION_NAME, self._ODS_REGIONS, self._ODS_OPERAND_SEGMENTS, self._ODS_RESULT_SEGMENTS, attributes=attributes, results=results, operands=operands, successors=_ods_successors, regions=regions, loc=loc, ip=ip)

@_ods_cext.register_op_adaptor(ApplyExpandStridedMetadataPatternsOp)
class ApplyExpandStridedMetadataPatternsOpAdaptor(_ods_ir.OpAdaptor):
  OPERATION_NAME = "transform.apply_patterns.memref.expand_strided_metadata"

def apply_patterns_memref_expand_strided_metadata(*, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None) -> ApplyExpandStridedMetadataPatternsOp:
  return ApplyExpandStridedMetadataPatternsOp(loc=loc, ip=ip)

@_ods_cext.register_operation(_Dialect)
class ApplyExtractAddressComputationsPatternsOp(_ods_ir.OpView):
  r"""
  Collects patterns for extracting address computations from operations
  with memory accesses such that these memory accesses use only a base
  pointer.
  
  For instance,
  ```mlir
  memref.load %base[%off0, ...]
  ```
  
  Will be rewritten in:
  ```mlir
  %new_base = memref.subview %base[%off0,...][1,...][1,...]
  memref.load %new_base[%c0,...]
  ```
  """

  OPERATION_NAME = "transform.apply_patterns.memref.extract_address_computations"

  _ODS_REGIONS = (0, True)

  def __init__(self, *, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None):
    operands = []
    attributes = {}
    regions = None
    _ods_context = _ods_get_default_loc_context(loc)
    results = []
    _ods_successors = None
    super().__init__(self.OPERATION_NAME, self._ODS_REGIONS, self._ODS_OPERAND_SEGMENTS, self._ODS_RESULT_SEGMENTS, attributes=attributes, results=results, operands=operands, successors=_ods_successors, regions=regions, loc=loc, ip=ip)

@_ods_cext.register_op_adaptor(ApplyExtractAddressComputationsPatternsOp)
class ApplyExtractAddressComputationsPatternsOpAdaptor(_ods_ir.OpAdaptor):
  OPERATION_NAME = "transform.apply_patterns.memref.extract_address_computations"

def apply_patterns_memref_extract_address_computations(*, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None) -> ApplyExtractAddressComputationsPatternsOp:
  return ApplyExtractAddressComputationsPatternsOp(loc=loc, ip=ip)

@_ods_cext.register_operation(_Dialect)
class ApplyFoldMemrefAliasOpsPatternsOp(_ods_ir.OpView):
  r"""
  Collects patterns for folding memref aliasing ops (memref.subview) into
  consumer load/store ops (affine.load, memref.load, nvgpu.ldmatrix,
  vector.load, vector.transfer_read, affine.store, memref.store, etc.) and
  other ops (e.g., memref.subview).
  """

  OPERATION_NAME = "transform.apply_patterns.memref.fold_memref_alias_ops"

  _ODS_REGIONS = (0, True)

  def __init__(self, *, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None):
    operands = []
    attributes = {}
    regions = None
    _ods_context = _ods_get_default_loc_context(loc)
    results = []
    _ods_successors = None
    super().__init__(self.OPERATION_NAME, self._ODS_REGIONS, self._ODS_OPERAND_SEGMENTS, self._ODS_RESULT_SEGMENTS, attributes=attributes, results=results, operands=operands, successors=_ods_successors, regions=regions, loc=loc, ip=ip)

@_ods_cext.register_op_adaptor(ApplyFoldMemrefAliasOpsPatternsOp)
class ApplyFoldMemrefAliasOpsPatternsOpAdaptor(_ods_ir.OpAdaptor):
  OPERATION_NAME = "transform.apply_patterns.memref.fold_memref_alias_ops"

def apply_patterns_memref_fold_memref_alias_ops(*, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None) -> ApplyFoldMemrefAliasOpsPatternsOp:
  return ApplyFoldMemrefAliasOpsPatternsOp(loc=loc, ip=ip)

@_ods_cext.register_operation(_Dialect)
class ApplyResolveRankedShapedTypeResultDimsPatternsOp(_ods_ir.OpView):
  r"""
  Collects patterns that resolve `memref.dim` operations with values that are
  defined by operations that implement the `ReifyRankedShapedTypeOpInterface`,
  in terms of shapes of its input operands.
  """

  OPERATION_NAME = "transform.apply_patterns.memref.resolve_ranked_shaped_type_result_dims"

  _ODS_REGIONS = (0, True)

  def __init__(self, *, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None):
    operands = []
    attributes = {}
    regions = None
    _ods_context = _ods_get_default_loc_context(loc)
    results = []
    _ods_successors = None
    super().__init__(self.OPERATION_NAME, self._ODS_REGIONS, self._ODS_OPERAND_SEGMENTS, self._ODS_RESULT_SEGMENTS, attributes=attributes, results=results, operands=operands, successors=_ods_successors, regions=regions, loc=loc, ip=ip)

@_ods_cext.register_op_adaptor(ApplyResolveRankedShapedTypeResultDimsPatternsOp)
class ApplyResolveRankedShapedTypeResultDimsPatternsOpAdaptor(_ods_ir.OpAdaptor):
  OPERATION_NAME = "transform.apply_patterns.memref.resolve_ranked_shaped_type_result_dims"

def apply_patterns_memref_resolve_ranked_shaped_type_result_dims(*, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None) -> ApplyResolveRankedShapedTypeResultDimsPatternsOp:
  return ApplyResolveRankedShapedTypeResultDimsPatternsOp(loc=loc, ip=ip)

@_ods_cext.register_operation(_Dialect)
class MemRefAllocaToGlobalOp(_ods_ir.OpView):
  r"""
  Inserts a new `memref.global` for each provided `memref.alloca` into the
  nearest symbol table (e.g., a `builtin.module`) and replaces it with a
  `memref.get_global`. This is useful, for example, for allocations that
  should reside in the shared memory of a GPU, which have to be declared as
  globals.
  
  #### Example
  
  Consider the following transform op:
  
  ```mlir
  %get_global, %global =
      transform.memref.alloca_to_global %alloca
        : (!transform.op<"memref.alloca">)
          -> (!transform.any_op, !transform.any_op)
  ```
  
  and the following input payload:
  
  ```mlir
  module {
    func.func @func() {
      %alloca = memref.alloca() : memref<2x32xf32>
      // usages of %alloca...
    }
  }
  ```
  
  then applying the transform op to the payload would result in the following
  output IR:
  
  ```mlir
  module {
    memref.global "private" @alloc : memref<2x32xf32>
    func.func @func() {
      %alloca = memref.get_global @alloc : memref<2x32xf32>
      // usages of %alloca...
    }
  }
  ```
  
  #### Return modes
  
  Succeeds always. The returned handles refer to the `memref.get_global` and
  `memref.global` ops that were inserted by the transformation.
  """

  OPERATION_NAME = "transform.memref.alloca_to_global"

  _ODS_REGIONS = (0, True)

  def __init__(self, getGlobal: _ods_ir.Type, global_: _ods_ir.Type, alloca: _ods_ir.Value, *, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None):
    operands = []
    attributes = {}
    regions = None
    operands.append(alloca)
    _ods_context = _ods_get_default_loc_context(loc)
    results = []
    results.append(getGlobal)
    results.append(global_)
    _ods_successors = None
    super().__init__(self.OPERATION_NAME, self._ODS_REGIONS, self._ODS_OPERAND_SEGMENTS, self._ODS_RESULT_SEGMENTS, attributes=attributes, results=results, operands=operands, successors=_ods_successors, regions=regions, loc=loc, ip=ip)

  @builtins.property
  def alloca(self) -> _ods_ir.Value:
    return self.operation.operands[0]

  @builtins.property
  def getGlobal(self) -> _ods_ir.OpResult:
    return self.operation.results[0]

  @builtins.property
  def global_(self) -> _ods_ir.OpResult:
    return self.operation.results[1]

@_ods_cext.register_op_adaptor(MemRefAllocaToGlobalOp)
class MemRefAllocaToGlobalOpAdaptor(_ods_ir.OpAdaptor):
  OPERATION_NAME = "transform.memref.alloca_to_global"

  @builtins.property
  def alloca(self) -> _ods_ir.Value:
    return self.operands[0]

def memref_alloca_to_global(get_global: _ods_ir.Type, global_: _ods_ir.Type, alloca: _ods_ir.Value, *, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None) -> _ods_ir.OpResultList:
  return MemRefAllocaToGlobalOp(getGlobal=get_global, global_=global_, alloca=alloca, loc=loc, ip=ip).results

@_ods_cext.register_operation(_Dialect)
class MemRefEraseDeadAllocAndStoresOp(_ods_ir.OpView):
  r"""
  This applies memory optimization on memref. In particular it does store to
  load forwarding, dead store elimination and dead alloc/alloca elimination.
  
  #### Return modes
  
  This operation applies a set of memory optimization on the whole region of
  the operand.
  
  The transformation does not consume the target handle. It modifies the
  payload. Dead allocations, loads and stores are silently dropped from all
  mappings.
  """

  OPERATION_NAME = "transform.memref.erase_dead_alloc_and_stores"

  _ODS_REGIONS = (0, True)

  def __init__(self, target: _ods_ir.Value, *, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None):
    operands = []
    attributes = {}
    regions = None
    operands.append(target)
    _ods_context = _ods_get_default_loc_context(loc)
    results = []
    _ods_successors = None
    super().__init__(self.OPERATION_NAME, self._ODS_REGIONS, self._ODS_OPERAND_SEGMENTS, self._ODS_RESULT_SEGMENTS, attributes=attributes, results=results, operands=operands, successors=_ods_successors, regions=regions, loc=loc, ip=ip)

  @builtins.property
  def target(self) -> _ods_ir.Value:
    return self.operation.operands[0]

@_ods_cext.register_op_adaptor(MemRefEraseDeadAllocAndStoresOp)
class MemRefEraseDeadAllocAndStoresOpAdaptor(_ods_ir.OpAdaptor):
  OPERATION_NAME = "transform.memref.erase_dead_alloc_and_stores"

  @builtins.property
  def target(self) -> _ods_ir.Value:
    return self.operands[0]

def memref_erase_dead_alloc_and_stores(target: _ods_ir.Value, *, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None) -> MemRefEraseDeadAllocAndStoresOp:
  return MemRefEraseDeadAllocAndStoresOp(target=target, loc=loc, ip=ip)

@_ods_cext.register_operation(_Dialect)
class MemRefMakeLoopIndependentOp(_ods_ir.OpView):
  r"""
  Rewrite the targeted ops such that their index-typed operands no longer
  depend on any loop induction variable of the `num_loop` enclosing `scf.for`
  loops. I.e., compute an upper bound that is independent of any such loop IV
  for every tensor dimension. The transformed op could then be hoisted from
  the `num_loop` enclosing loops. To preserve the original semantics, place a
  `memref.subview` inside the loop.
  
  Currently supported operations are:
  - memref.alloca: Replaced with a new memref.alloca with upper bound sizes,
    followed by a memref.subview.
  
  #### Return modes
  
  This operation fails if at least one induction variable could not be
  eliminated. In case the targeted op is already independent of induction
  variables, this transform succeeds and returns the unmodified target op.
  
  Otherwise, the returned handle points to a subset of the produced ops:
  - memref.alloca: The returned handle points to the memref.subview op.
  
  This transform op consumes the target handle and produces a result handle.
  """

  OPERATION_NAME = "transform.memref.make_loop_independent"

  _ODS_REGIONS = (0, True)

  def __init__(self, transformed: _ods_ir.Type, target: _ods_ir.Value, num_loops: _Union[int, _ods_ir.IntegerAttr], *, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None):
    operands = []
    attributes = {}
    regions = None
    operands.append(target)
    _ods_context = _ods_get_default_loc_context(loc)
    attributes["num_loops"] = (num_loops if (
    isinstance(num_loops, _ods_ir.Attribute) or
    not _ods_ir.AttrBuilder.contains('I64Attr')) else
      _ods_ir.AttrBuilder.get('I64Attr')(num_loops, context=_ods_context))
    results = []
    results.append(transformed)
    _ods_successors = None
    super().__init__(self.OPERATION_NAME, self._ODS_REGIONS, self._ODS_OPERAND_SEGMENTS, self._ODS_RESULT_SEGMENTS, attributes=attributes, results=results, operands=operands, successors=_ods_successors, regions=regions, loc=loc, ip=ip)

  @builtins.property
  def target(self) -> _ods_ir.Value:
    return self.operation.operands[0]

  @builtins.property
  def num_loops(self) -> _ods_ir.IntegerAttr:
    return self.operation.attributes["num_loops"]

  @num_loops.setter
  def num_loops(self, value: _ods_ir.IntegerAttr):
    if value is None:
      raise ValueError("'None' not allowed as value for mandatory attributes")
    self.operation.attributes["num_loops"] = value

  @builtins.property
  def transformed(self) -> _ods_ir.OpResult:
    return self.operation.results[0]

@_ods_cext.register_op_adaptor(MemRefMakeLoopIndependentOp)
class MemRefMakeLoopIndependentOpAdaptor(_ods_ir.OpAdaptor):
  OPERATION_NAME = "transform.memref.make_loop_independent"

  @builtins.property
  def target(self) -> _ods_ir.Value:
    return self.operands[0]

  @builtins.property
  def num_loops(self) -> _ods_ir.IntegerAttr:
    return self.attributes["num_loops"]

def memref_make_loop_independent(transformed: _ods_ir.Type, target: _ods_ir.Value, num_loops: _Union[int, _ods_ir.IntegerAttr], *, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None) -> _ods_ir.OpResult:
  return MemRefMakeLoopIndependentOp(transformed=transformed, target=target, num_loops=num_loops, loc=loc, ip=ip).result

@_ods_cext.register_operation(_Dialect)
class MemRefMultiBufferOp(_ods_ir.OpView):
  r"""
  Transformation to do multi-buffering/array expansion to remove
  dependencies on the temporary allocation between consecutive loop
  iterations. This transform expands the size of an allocation by
  a given multiplicative factor and fixes up any users of the
  multibuffered allocation.
  If skip analysis is not set the transformation will only apply
  if it can prove that there is no data being carried across loop
  iterations.
  
  #### Return modes
  
  This operation returns the new allocation if multi-buffering
  succeeds, and failure otherwise.
  """

  OPERATION_NAME = "transform.memref.multibuffer"

  _ODS_REGIONS = (0, True)

  def __init__(self, transformed: _ods_ir.Type, target: _ods_ir.Value, factor: _Union[int, _ods_ir.IntegerAttr], *, skip_analysis: _Optional[bool] = None, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None):
    operands = []
    attributes = {}
    regions = None
    operands.append(target)
    _ods_context = _ods_get_default_loc_context(loc)
    attributes["factor"] = (factor if (
    isinstance(factor, _ods_ir.Attribute) or
    not _ods_ir.AttrBuilder.contains('I64Attr')) else
      _ods_ir.AttrBuilder.get('I64Attr')(factor, context=_ods_context))
    if bool(skip_analysis): attributes["skip_analysis"] = _ods_ir.UnitAttr.get(
      _ods_get_default_loc_context(loc))
    results = []
    results.append(transformed)
    _ods_successors = None
    super().__init__(self.OPERATION_NAME, self._ODS_REGIONS, self._ODS_OPERAND_SEGMENTS, self._ODS_RESULT_SEGMENTS, attributes=attributes, results=results, operands=operands, successors=_ods_successors, regions=regions, loc=loc, ip=ip)

  @builtins.property
  def target(self) -> _ods_ir.Value:
    return self.operation.operands[0]

  @builtins.property
  def factor(self) -> _ods_ir.IntegerAttr:
    return self.operation.attributes["factor"]

  @factor.setter
  def factor(self, value: _ods_ir.IntegerAttr):
    if value is None:
      raise ValueError("'None' not allowed as value for mandatory attributes")
    self.operation.attributes["factor"] = value

  @builtins.property
  def skip_analysis(self) -> bool:
    return "skip_analysis" in self.operation.attributes

  @skip_analysis.setter
  def skip_analysis(self, value):
    if bool(value):
      self.operation.attributes["skip_analysis"] = _ods_ir.UnitAttr.get()
    elif "skip_analysis" in self.operation.attributes:
      del self.operation.attributes["skip_analysis"]

  @skip_analysis.deleter
  def skip_analysis(self):
    del self.operation.attributes["skip_analysis"]

  @builtins.property
  def transformed(self) -> _ods_ir.OpResult:
    return self.operation.results[0]

@_ods_cext.register_op_adaptor(MemRefMultiBufferOp)
class MemRefMultiBufferOpAdaptor(_ods_ir.OpAdaptor):
  OPERATION_NAME = "transform.memref.multibuffer"

  @builtins.property
  def target(self) -> _ods_ir.Value:
    return self.operands[0]

  @builtins.property
  def factor(self) -> _ods_ir.IntegerAttr:
    return self.attributes["factor"]

  @builtins.property
  def skip_analysis(self) -> bool:
    return "skip_analysis" in self.attributes

def memref_multibuffer(transformed: _ods_ir.Type, target: _ods_ir.Value, factor: _Union[int, _ods_ir.IntegerAttr], *, skip_analysis: _Optional[bool] = None, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None) -> _ods_ir.OpResult:
  return MemRefMultiBufferOp(transformed=transformed, target=target, factor=factor, skip_analysis=skip_analysis, loc=loc, ip=ip).result

@_ods_cext.register_operation(_Dialect)
class MemrefToLLVMTypeConverterOp(_ods_ir.OpView):
  r"""
  This operation provides an "LLVMTypeConverter" that lowers memref types to
  LLVM types.
  
  The type converter can be customized as follows:
  - `use_aligned_alloc`: Use aligned_alloc in place of malloc for heap
    allocations.
  - `index_bitwidth`: Bitwidth of the index type, "0" indicates the size of a
    machine word.
  - `use_generic_functions`: Use generic allocation and deallocation functions
    instead of the classic "malloc", "aligned_alloc" and "free" functions.
  // TODO: the following two options don't really make sense for 
  // memref_to_llvm_type_converter specifically.
  // We should have a single to_llvm_type_converter.
  - `use_bare_ptr_call_conv`: Replace FuncOp's MemRef arguments with bare 
    pointers to the MemRef element types.
  - `data-layout`: String description (LLVM format) of the data layout that is
    expected on the produced module.
  """

  OPERATION_NAME = "transform.apply_conversion_patterns.memref.memref_to_llvm_type_converter"

  _ODS_REGIONS = (0, True)

  def __init__(self, *, use_aligned_alloc: _Optional[_Union[bool, _ods_ir.BoolAttr]] = None, index_bitwidth: _Optional[_Union[int, _ods_ir.IntegerAttr]] = None, use_generic_functions: _Optional[_Union[bool, _ods_ir.BoolAttr]] = None, use_bare_ptr_call_conv: _Optional[_Union[bool, _ods_ir.BoolAttr]] = None, data_layout: _Optional[_Union[str, _ods_ir.StringAttr]] = None, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None):
    operands = []
    attributes = {}
    regions = None
    _ods_context = _ods_get_default_loc_context(loc)
    if use_aligned_alloc is not None: attributes["use_aligned_alloc"] = (use_aligned_alloc if (
        isinstance(use_aligned_alloc, _ods_ir.Attribute) or
        not _ods_ir.AttrBuilder.contains('BoolAttr')) else
          _ods_ir.AttrBuilder.get('BoolAttr')(use_aligned_alloc, context=_ods_context))
    if index_bitwidth is not None: attributes["index_bitwidth"] = (index_bitwidth if (
        isinstance(index_bitwidth, _ods_ir.Attribute) or
        not _ods_ir.AttrBuilder.contains('I64Attr')) else
          _ods_ir.AttrBuilder.get('I64Attr')(index_bitwidth, context=_ods_context))
    if use_generic_functions is not None: attributes["use_generic_functions"] = (use_generic_functions if (
        isinstance(use_generic_functions, _ods_ir.Attribute) or
        not _ods_ir.AttrBuilder.contains('BoolAttr')) else
          _ods_ir.AttrBuilder.get('BoolAttr')(use_generic_functions, context=_ods_context))
    if use_bare_ptr_call_conv is not None: attributes["use_bare_ptr_call_conv"] = (use_bare_ptr_call_conv if (
        isinstance(use_bare_ptr_call_conv, _ods_ir.Attribute) or
        not _ods_ir.AttrBuilder.contains('BoolAttr')) else
          _ods_ir.AttrBuilder.get('BoolAttr')(use_bare_ptr_call_conv, context=_ods_context))
    if data_layout is not None: attributes["data_layout"] = (data_layout if (
        isinstance(data_layout, _ods_ir.Attribute) or
        not _ods_ir.AttrBuilder.contains('StrAttr')) else
          _ods_ir.AttrBuilder.get('StrAttr')(data_layout, context=_ods_context))
    results = []
    _ods_successors = None
    super().__init__(self.OPERATION_NAME, self._ODS_REGIONS, self._ODS_OPERAND_SEGMENTS, self._ODS_RESULT_SEGMENTS, attributes=attributes, results=results, operands=operands, successors=_ods_successors, regions=regions, loc=loc, ip=ip)

  @builtins.property
  def use_aligned_alloc(self) -> _Optional[_ods_ir.BoolAttr]:
    if "use_aligned_alloc" not in self.operation.attributes:
      return None
    return self.operation.attributes["use_aligned_alloc"]

  @use_aligned_alloc.setter
  def use_aligned_alloc(self, value: _Optional[_ods_ir.BoolAttr]):
    if value is not None:
      self.operation.attributes["use_aligned_alloc"] = value
    elif "use_aligned_alloc" in self.operation.attributes:
      del self.operation.attributes["use_aligned_alloc"]

  @use_aligned_alloc.deleter
  def use_aligned_alloc(self):
    del self.operation.attributes["use_aligned_alloc"]

  @builtins.property
  def index_bitwidth(self) -> _Optional[_ods_ir.IntegerAttr]:
    if "index_bitwidth" not in self.operation.attributes:
      return None
    return self.operation.attributes["index_bitwidth"]

  @index_bitwidth.setter
  def index_bitwidth(self, value: _Optional[_ods_ir.IntegerAttr]):
    if value is not None:
      self.operation.attributes["index_bitwidth"] = value
    elif "index_bitwidth" in self.operation.attributes:
      del self.operation.attributes["index_bitwidth"]

  @index_bitwidth.deleter
  def index_bitwidth(self):
    del self.operation.attributes["index_bitwidth"]

  @builtins.property
  def use_generic_functions(self) -> _Optional[_ods_ir.BoolAttr]:
    if "use_generic_functions" not in self.operation.attributes:
      return None
    return self.operation.attributes["use_generic_functions"]

  @use_generic_functions.setter
  def use_generic_functions(self, value: _Optional[_ods_ir.BoolAttr]):
    if value is not None:
      self.operation.attributes["use_generic_functions"] = value
    elif "use_generic_functions" in self.operation.attributes:
      del self.operation.attributes["use_generic_functions"]

  @use_generic_functions.deleter
  def use_generic_functions(self):
    del self.operation.attributes["use_generic_functions"]

  @builtins.property
  def use_bare_ptr_call_conv(self) -> _Optional[_ods_ir.BoolAttr]:
    if "use_bare_ptr_call_conv" not in self.operation.attributes:
      return None
    return self.operation.attributes["use_bare_ptr_call_conv"]

  @use_bare_ptr_call_conv.setter
  def use_bare_ptr_call_conv(self, value: _Optional[_ods_ir.BoolAttr]):
    if value is not None:
      self.operation.attributes["use_bare_ptr_call_conv"] = value
    elif "use_bare_ptr_call_conv" in self.operation.attributes:
      del self.operation.attributes["use_bare_ptr_call_conv"]

  @use_bare_ptr_call_conv.deleter
  def use_bare_ptr_call_conv(self):
    del self.operation.attributes["use_bare_ptr_call_conv"]

  @builtins.property
  def data_layout(self) -> _Optional[_ods_ir.StringAttr]:
    if "data_layout" not in self.operation.attributes:
      return None
    return self.operation.attributes["data_layout"]

  @data_layout.setter
  def data_layout(self, value: _Optional[_ods_ir.StringAttr]):
    if value is not None:
      self.operation.attributes["data_layout"] = value
    elif "data_layout" in self.operation.attributes:
      del self.operation.attributes["data_layout"]

  @data_layout.deleter
  def data_layout(self):
    del self.operation.attributes["data_layout"]

@_ods_cext.register_op_adaptor(MemrefToLLVMTypeConverterOp)
class MemrefToLLVMTypeConverterOpAdaptor(_ods_ir.OpAdaptor):
  OPERATION_NAME = "transform.apply_conversion_patterns.memref.memref_to_llvm_type_converter"

  @builtins.property
  def use_aligned_alloc(self) -> _Optional[_ods_ir.BoolAttr]:
    if "use_aligned_alloc" not in self.attributes:
      return None
    return self.attributes["use_aligned_alloc"]

  @builtins.property
  def index_bitwidth(self) -> _Optional[_ods_ir.IntegerAttr]:
    if "index_bitwidth" not in self.attributes:
      return None
    return self.attributes["index_bitwidth"]

  @builtins.property
  def use_generic_functions(self) -> _Optional[_ods_ir.BoolAttr]:
    if "use_generic_functions" not in self.attributes:
      return None
    return self.attributes["use_generic_functions"]

  @builtins.property
  def use_bare_ptr_call_conv(self) -> _Optional[_ods_ir.BoolAttr]:
    if "use_bare_ptr_call_conv" not in self.attributes:
      return None
    return self.attributes["use_bare_ptr_call_conv"]

  @builtins.property
  def data_layout(self) -> _Optional[_ods_ir.StringAttr]:
    if "data_layout" not in self.attributes:
      return None
    return self.attributes["data_layout"]

def apply_conversion_patterns_memref_memref_to_llvm_type_converter(*, use_aligned_alloc: _Optional[_Union[bool, _ods_ir.BoolAttr]] = None, index_bitwidth: _Optional[_Union[int, _ods_ir.IntegerAttr]] = None, use_generic_functions: _Optional[_Union[bool, _ods_ir.BoolAttr]] = None, use_bare_ptr_call_conv: _Optional[_Union[bool, _ods_ir.BoolAttr]] = None, data_layout: _Optional[_Union[str, _ods_ir.StringAttr]] = None, loc: _Optional[_ods_ir.Location] = None, ip: _Optional[_ods_ir.InsertionPoint] = None) -> MemrefToLLVMTypeConverterOp:
  return MemrefToLLVMTypeConverterOp(use_aligned_alloc=use_aligned_alloc, index_bitwidth=index_bitwidth, use_generic_functions=use_generic_functions, use_bare_ptr_call_conv=use_bare_ptr_call_conv, data_layout=data_layout, loc=loc, ip=ip)



# SPDX-FileCopyrightText: Copyright (c) 2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
"""Vector load/store operations for device code."""


def load(
    array: "MemrefLike",
    indices: tuple[int, ...] | int,
    shape: tuple[int, ...] | int,
    alignment: int | None = None,
) -> "Vector":
    """Load a vector from array at index.

    Args:
        array: Source array (memref) to load from
        indices: Starting index - can be int for 1D arrays or tuple for N-D arrays
        shape: Vector shape - int for 1D vectors or tuple for N-D vectors
        alignment: Optional memory alignment in bytes. When provided, generates
                   llvm.load with vector type and alignment attribute, which LLVM
                   optimizes to vectorized PTX instructions (e.g., ld.global.v4.b16).
                   Common values: 4, 8, 16 bytes.
                   Default None uses vector.transfer_read (may be scalarized).

    Returns:
        Vector of the specified shape loaded from the array

    Examples:
        # 1D vector load without alignment (uses vector.transfer_read)
        vec = cuda.vector.load(arr, i, 4)

        # 1D vector load with alignment (uses llvm.load → vectorized PTX)
        vec = cuda.vector.load(arr, i, 4, alignment=8)  # 4 x fp16 = 8 bytes

        # 2D array with 1D vector load
        vec = cuda.vector.load(arr_2d, (row, col), 4, alignment=16)

        # Alignment guidelines:
        # - 4 x fp16 (half): alignment=8  (4 * 2 bytes)
        # - 8 x fp16 (half): alignment=16 (8 * 2 bytes)
        # - 4 x fp32 (float): alignment=16 (4 * 4 bytes)
    """
    ...


def store(
    array: "MemrefLike",
    index: tuple[int, ...] | int,
    vector: "Vector",
    alignment: int | None = None,
) -> None:
    """Store a vector to array starting at index.

    Args:
        array: Destination array (memref) to store into
        index: Starting index - can be int for 1D arrays or tuple for N-D arrays
        vector: Vector value to store
        alignment: Optional memory alignment in bytes. When provided, generates
                   llvm.store with alignment attribute, which LLVM optimizes to
                   vectorized PTX instructions (e.g., st.global.v4.b16).
                   Must match alignment used in corresponding load.
                   Default None uses vector.transfer_write (may be scalarized).

    Examples:
        # 1D vector store without alignment
        cuda.vector.store(arr, i, vec)

        # 1D vector store with alignment (uses llvm.store → vectorized PTX)
        cuda.vector.store(arr, i, vec, alignment=8)  # 4 x fp16 = 8 bytes

        # 2D array with 1D vector store
        cuda.vector.store(arr_2d, (row, col), vec, alignment=16)

        # Note: alignment should match the actual memory alignment of the data.
    """
    ...



# SPDX-FileCopyrightText: Copyright (c) 2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# SPDX-License-Identifier: BSD-2-Clause

"""This module provides the unsafe things for targets/numbers.py"""

from numba_cuda_mlir.numba_cuda import types
from numba_cuda_mlir.numba_cuda.core import errors
from numba_cuda_mlir.numba_cuda.extending import intrinsic


# These intrinsics' codegen closures built llvmlite bitcast/cttz/ctlz IR. They
# are filtered out on the MLIR path, so the codegen is a shared tombstone; the
# typing (returned signatures and validation) is retained.
def _dead_codegen(context, builder, signature, args):
    raise NotImplementedError(
        "this intrinsic's vendored llvmlite codegen is not used on the MLIR path"
    )


@intrinsic
def viewer(tyctx, val, viewty):
    """Bitcast a scalar 'val' to the given type 'viewty'."""
    retty = viewty.dtype
    sig = retty(val, viewty)
    return sig, _dead_codegen


@intrinsic
def trailing_zeros(typeingctx, src):
    """Counts trailing zeros in the binary representation of an integer."""
    if not isinstance(src, types.Integer):
        msg = f"trailing_zeros is only defined for integers, but value passed was '{src}'."
        raise errors.NumbaTypeError(msg)

    return src(src), _dead_codegen


@intrinsic
def leading_zeros(typeingctx, src):
    """Counts leading zeros in the binary representation of an integer."""
    if not isinstance(src, types.Integer):
        msg = f"leading_zeros is only defined for integers, but value passed was '{src}'."
        raise errors.NumbaTypeError(msg)

    return src(src), _dead_codegen



# SPDX-FileCopyrightText: Copyright (c) 2026 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
from numba_cuda_mlir.types import float16, boolean


def habs(arg0: float16) -> float16: ...


def hadd(arg0: float16, arg1: float16) -> float16: ...


def hceil(arg0: float16) -> float16: ...


def hcos(arg0: float16) -> float16: ...


def hdiv(arg0: float16, arg1: float16) -> float16: ...


def heq(arg0: float16, arg1: float16) -> boolean: ...


def hexp(arg0: float16) -> float16: ...


def hexp10(arg0: float16) -> float16: ...


def hexp2(arg0: float16) -> float16: ...


def hfloor(arg0: float16) -> float16: ...


def hfma(arg0: float16, arg1: float16, arg2: float16) -> float16: ...


def hge(arg0: float16, arg1: float16) -> boolean: ...


def hgt(arg0: float16, arg1: float16) -> boolean: ...


def hle(arg0: float16, arg1: float16) -> boolean: ...


def hlog(arg0: float16) -> float16: ...


def hlog10(arg0: float16) -> float16: ...


def hlog2(arg0: float16) -> float16: ...


def hlt(arg0: float16, arg1: float16) -> boolean: ...


def hmax(arg0: float16, arg1: float16) -> float16: ...


def hmin(arg0: float16, arg1: float16) -> float16: ...


def hmul(arg0: float16, arg1: float16) -> float16: ...


def hne(arg0: float16, arg1: float16) -> boolean: ...


def hneg(arg0: float16) -> float16: ...


def hrcp(arg0: float16) -> float16: ...


def hrint(arg0: float16) -> float16: ...


def hrsqrt(arg0: float16) -> float16: ...


def hsin(arg0: float16) -> float16: ...


def hsqrt(arg0: float16) -> float16: ...


def hsub(arg0: float16, arg1: float16) -> float16: ...


def htanh(arg0: float16) -> float16: ...


def htanh_approx(arg0: float16) -> float16: ...


def htrunc(arg0: float16) -> float16: ...
